@@ -14,21 +14,26 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <config-libkleo.h>
+
 #include "keyresolvercore.h"
 
-#include "kleo/enum.h"
-#include "kleo/keygroup.h"
-#include "models/keycache.h"
-#include "utils/formatting.h"
+#include "enum.h"
+#include "keygroup.h"
+
+#include <libkleo/formatting.h>
+#include <libkleo/gnupg.h>
+#include <libkleo/keycache.h>
+
+#include <libkleo_debug.h>
 
 #include <gpgme++/key.h>
 
-#include "libkleo_debug.h"
-
 using namespace Kleo;
 using namespace GpgME;
 
-namespace {
+namespace
+{
 
 QDebug operator<<(QDebug debug, const GpgME::Key &key)
 {
@@ -42,8 +47,7 @@ QDebug operator<<(QDebug debug, const GpgME::Key &key)
 
 static inline bool ValidEncryptionKey(const Key &key)
 {
-    if (key.isNull() || key.isRevoked() || key.isExpired() ||
-        key.isDisabled() || !key.canEncrypt()) {
+    if (key.isNull() || key.isRevoked() || key.isExpired() || key.isDisabled() || !key.canEncrypt()) {
         return false;
     }
     return true;
@@ -51,8 +55,7 @@ static inline bool ValidEncryptionKey(const Key &key)
 
 static inline bool ValidSigningKey(const Key &key)
 {
-    if (key.isNull() || key.isRevoked() || key.isExpired() ||
-        key.isDisabled() || !key.canSign() || !key.hasSecret()) {
+    if (key.isNull() || key.isRevoked() || key.isExpired() || key.isDisabled() || !key.canSign() || !key.hasSecret()) {
         return false;
     }
     return true;
@@ -62,7 +65,7 @@ static int keyValidity(const Key &key, const QString &address)
 {
     // returns the validity of the UID matching the address or, if no UID matches, the maximal validity of all UIDs
     int overallValidity = UserID::Validity::Unknown;
-    for (const auto &uid: key.userIDs()) {
+    for (const auto &uid : key.userIDs()) {
         if (QString::fromStdString(uid.addrSpec()).toLower() == address.toLower()) {
             return uid.validity();
         }
@@ -73,8 +76,10 @@ static int keyValidity(const Key &key, const QString &address)
 
 static int minimumValidity(const std::vector<Key> &keys, const QString &address)
 {
-    const int minValidity = std::accumulate(keys.cbegin(), keys.cend(), UserID::Ultimate + 1,
-                                            [address] (int validity, const Key &key) {
+    const int minValidity = std::accumulate(keys.cbegin(), //
+                                            keys.cend(),
+                                            UserID::Ultimate + 1,
+                                            [address](int validity, const Key &key) {
                                                 return std::min<int>(validity, keyValidity(key, address));
                                             });
     return minValidity <= UserID::Ultimate ? static_cast<UserID::Validity>(minValidity) : UserID::Unknown;
@@ -82,12 +87,16 @@ static int minimumValidity(const std::vector<Key> &keys, const QString &address)
 
 bool allKeysHaveProtocol(const std::vector<Key> &keys, Protocol protocol)
 {
-    return std::all_of(keys.cbegin(), keys.cend(), [protocol] (const Key &key) { return key.protocol() == protocol; });
+    return std::all_of(keys.cbegin(), keys.cend(), [protocol](const Key &key) {
+        return key.protocol() == protocol;
+    });
 }
 
 bool anyKeyHasProtocol(const std::vector<Key> &keys, Protocol protocol)
 {
-    return std::any_of(std::begin(keys), std::end(keys), [protocol] (const Key &key) { return key.protocol() == protocol; });
+    return std::any_of(std::begin(keys), std::end(keys), [protocol](const Key &key) {
+        return key.protocol() == protocol;
+    });
 }
 
 } // namespace
@@ -95,7 +104,7 @@ bool anyKeyHasProtocol(const std::vector<Key> &keys, Protocol protocol)
 class KeyResolverCore::Private
 {
 public:
-    Private(KeyResolverCore* qq, bool enc, bool sig, Protocol fmt)
+    Private(KeyResolverCore *qq, bool enc, bool sig, Protocol fmt)
         : q(qq)
         , mFormat(fmt)
         , mEncrypt(enc)
@@ -103,7 +112,6 @@ public:
         , mCache(KeyCache::instance())
         , mPreferredProtocol(UnknownProtocol)
         , mMinimumValidity(UserID::Marginal)
-        , mCompliance(Formatting::complianceMode())
     {
     }
 
@@ -143,7 +151,6 @@ public:
     bool mAllowMixed = true;
     Protocol mPreferredProtocol;
     int mMinimumValidity;
-    QString mCompliance;
 };
 
 bool KeyResolverCore::Private::isAcceptableSigningKey(const Key &key)
@@ -151,10 +158,9 @@ bool KeyResolverCore::Private::isAcceptableSigningKey(const Key &key)
     if (!ValidSigningKey(key)) {
         return false;
     }
-    if (mCompliance == QLatin1String("de-vs")) {
+    if (Kleo::gnupgIsDeVsCompliant()) {
         if (!Formatting::isKeyDeVs(key)) {
-            qCDebug(LIBKLEO_LOG) << "Rejected sig key" << key.primaryFingerprint()
-                                    << "because it is not de-vs compliant.";
+            qCDebug(LIBKLEO_LOG) << "Rejected sig key" << key.primaryFingerprint() << "because it is not de-vs compliant.";
             return false;
         }
     }
@@ -167,10 +173,9 @@ bool KeyResolverCore::Private::isAcceptableEncryptionKey(const Key &key, const Q
         return false;
     }
 
-    if (mCompliance == QLatin1String("de-vs")) {
+    if (Kleo::gnupgIsDeVsCompliant()) {
         if (!Formatting::isKeyDeVs(key)) {
-            qCDebug(LIBKLEO_LOG) << "Rejected enc key" << key.primaryFingerprint()
-                                    << "because it is not de-vs compliant.";
+            qCDebug(LIBKLEO_LOG) << "Rejected enc key" << key.primaryFingerprint() << "because it is not de-vs compliant.";
             return false;
         }
     }
@@ -178,7 +183,7 @@ bool KeyResolverCore::Private::isAcceptableEncryptionKey(const Key &key, const Q
     if (address.isEmpty()) {
         return true;
     }
-    for (const auto &uid: key.userIDs()) {
+    for (const auto &uid : key.userIDs()) {
         if (uid.addrSpec() == address.toStdString()) {
             if (uid.validity() >= mMinimumValidity) {
                 return true;
@@ -190,7 +195,7 @@ bool KeyResolverCore::Private::isAcceptableEncryptionKey(const Key &key, const Q
 
 void KeyResolverCore::Private::setSender(const QString &address)
 {
-    const auto normalized = UserID::addrSpecFromString (address.toUtf8().constData());
+    const auto normalized = UserID::addrSpecFromString(address.toUtf8().constData());
     if (normalized.empty()) {
         // should not happen bug in the caller, non localized
         // error for bug reporting.
@@ -210,9 +215,9 @@ void KeyResolverCore::Private::addRecipients(const QStringList &addresses)
 
     // Internally we work with normalized addresses. Normalization
     // matches the gnupg one.
-    for (const auto &addr: addresses) {
+    for (const auto &addr : addresses) {
         // PGP Uids are defined to be UTF-8 (RFC 4880 §5.11)
-        const auto normalized = UserID::addrSpecFromString (addr.toUtf8().constData());
+        const auto normalized = UserID::addrSpecFromString(addr.toUtf8().constData());
         if (normalized.empty()) {
             // should not happen bug in the caller, non localized
             // error for bug reporting.
@@ -247,11 +252,11 @@ namespace
 std::vector<Key> resolveOverride(const QString &address, Protocol protocol, const QStringList &fingerprints)
 {
     std::vector<Key> keys;
-    for (const auto &fprOrId: fingerprints) {
+    for (const auto &fprOrId : fingerprints) {
         const Key key = KeyCache::instance()->findByKeyIDOrFingerprint(fprOrId.toUtf8().constData());
         if (key.isNull()) {
             // FIXME: Report to caller
-            qCDebug (LIBKLEO_LOG) << "Failed to find override key for:" << address << "fpr:" << fprOrId;
+            qCDebug(LIBKLEO_LOG) << "Failed to find override key for:" << address << "fpr:" << fprOrId;
             continue;
         }
         if (protocol != UnknownProtocol && key.protocol() != protocol) {
@@ -278,7 +283,8 @@ void KeyResolverCore::Private::resolveOverrides()
 
         if (!mRecipients.contains(address)) {
             qCDebug(LIBKLEO_LOG) << "Overrides provided for an address that is "
-                "neither sender nor recipient. Address:" << address;
+                                    "neither sender nor recipient. Address:"
+                                 << address;
             continue;
         }
 
@@ -305,9 +311,9 @@ void KeyResolverCore::Private::resolveOverrides()
 std::vector<Key> KeyResolverCore::Private::resolveSenderWithGroup(const QString &address, Protocol protocol)
 {
     // prefer single-protocol groups over mixed-protocol groups
-    auto group = mCache->findGroup(address, protocol, KeyUsage::Sign);
+    auto group = mCache->findGroup(address, protocol, KeyCache::KeyUsage::Sign);
     if (group.isNull()) {
-        group = mCache->findGroup(address, UnknownProtocol, KeyUsage::Sign);
+        group = mCache->findGroup(address, UnknownProtocol, KeyCache::KeyUsage::Sign);
     }
     if (group.isNull()) {
         return {};
@@ -315,7 +321,9 @@ std::vector<Key> KeyResolverCore::Private::resolveSenderWithGroup(const QString
 
     // take the first key matching the protocol
     const auto &keys = group.keys();
-    const auto it = std::find_if(std::begin(keys), std::end(keys), [protocol] (const auto &key) { return key.protocol() == protocol; });
+    const auto it = std::find_if(std::begin(keys), std::end(keys), [protocol](const auto &key) {
+        return key.protocol() == protocol;
+    });
     if (it == std::end(keys)) {
         qCDebug(LIBKLEO_LOG) << "group" << group.name() << "has no" << Formatting::displayName(protocol) << "signing key";
         return {};
@@ -359,7 +367,7 @@ void KeyResolverCore::Private::resolveSign(Protocol proto)
         // Explicitly set
         return;
     }
-    const auto key = mCache->findBestByMailBox(mSender.toUtf8().constData(), proto, KeyUsage::Sign);
+    const auto key = mCache->findBestByMailBox(mSender.toUtf8().constData(), proto, KeyCache::KeyUsage::Sign);
     if (key.isNull()) {
         qCDebug(LIBKLEO_LOG) << "Failed to find" << Formatting::displayName(proto) << "signing key for" << mSender;
         return;
@@ -374,7 +382,7 @@ void KeyResolverCore::Private::resolveSign(Protocol proto)
 void KeyResolverCore::Private::setSigningKeys(const QStringList &fingerprints)
 {
     if (mSign) {
-        for (const auto &fpr: fingerprints) {
+        for (const auto &fpr : fingerprints) {
             const auto key = mCache->findByKeyIDOrFingerprint(fpr.toUtf8().constData());
             if (key.isNull()) {
                 qCDebug(LIBKLEO_LOG) << "Failed to find signing key with fingerprint" << fpr;
@@ -387,7 +395,7 @@ void KeyResolverCore::Private::setSigningKeys(const QStringList &fingerprints)
 
 std::vector<Key> KeyResolverCore::Private::resolveRecipientWithGroup(const QString &address, Protocol protocol)
 {
-    const auto group = mCache->findGroup(address, protocol, KeyUsage::Encrypt);
+    const auto group = mCache->findGroup(address, protocol, KeyCache::KeyUsage::Encrypt);
     if (group.isNull()) {
         return {};
     }
@@ -400,13 +408,14 @@ std::vector<Key> KeyResolverCore::Private::resolveRecipientWithGroup(const QStri
     // will also show unacceptable group keys so that the
     // user can see which key is not acceptable.
     const auto &keys = group.keys();
-    const bool allKeysAreAcceptable =
-        std::all_of(std::begin(keys), std::end(keys), [this] (const auto &key) { return isAcceptableEncryptionKey(key); });
+    const bool allKeysAreAcceptable = std::all_of(std::begin(keys), std::end(keys), [this](const auto &key) {
+        return isAcceptableEncryptionKey(key);
+    });
     if (!allKeysAreAcceptable) {
         qCDebug(LIBKLEO_LOG) << "group" << group.name() << "has at least one unacceptable key";
         return {};
     }
-    for (const auto &k: keys) {
+    for (const auto &k : keys) {
         qCDebug(LIBKLEO_LOG) << "Resolved encrypt to" << address << "with key" << k.primaryFingerprint();
     }
     std::vector<Key> result;
@@ -458,14 +467,13 @@ void KeyResolverCore::Private::resolveEncryptionGroups()
 
 std::vector<Key> KeyResolverCore::Private::resolveRecipient(const QString &address, Protocol protocol)
 {
-    const auto key = mCache->findBestByMailBox(address.toUtf8().constData(), protocol, KeyUsage::Encrypt);
+    const auto key = mCache->findBestByMailBox(address.toUtf8().constData(), protocol, KeyCache::KeyUsage::Encrypt);
     if (key.isNull()) {
         qCDebug(LIBKLEO_LOG) << "Failed to find any" << Formatting::displayName(protocol) << "key for:" << address;
         return {};
     }
     if (!isAcceptableEncryptionKey(key, address)) {
-        qCDebug(LIBKLEO_LOG) << "key for:" << address << key.primaryFingerprint()
-                             << "has not enough validity";
+        qCDebug(LIBKLEO_LOG) << "key for:" << address << key.primaryFingerprint() << "has not enough validity";
         return {};
     }
     qCDebug(LIBKLEO_LOG) << "Resolved encrypt to" << address << "with key" << key.primaryFingerprint();
@@ -534,20 +542,23 @@ auto getBestEncryptionKeys(const QMap<QString, QMap<Protocol, std::vector<Key>>>
 
 namespace
 {
+bool hasUnresolvedSender(const QMap<Protocol, std::vector<Key>> &signingKeys, Protocol protocol)
+{
+    return signingKeys.value(protocol).empty();
+}
+
 bool hasUnresolvedRecipients(const QMap<QString, QMap<Protocol, std::vector<Key>>> &encryptionKeys, Protocol protocol)
 {
-    return std::any_of(std::cbegin(encryptionKeys), std::cend(encryptionKeys),
-                       [protocol] (const auto &protocolKeysMap) {
-                           return protocolKeysMap.value(protocol).empty();
-                       });
+    return std::any_of(std::cbegin(encryptionKeys), std::cend(encryptionKeys), [protocol](const auto &protocolKeysMap) {
+        return protocolKeysMap.value(protocol).empty();
+    });
 }
 
 bool anyCommonOverrideHasKeyOfType(const QMap<QString, QMap<Protocol, std::vector<Key>>> &encryptionKeys, Protocol protocol)
 {
-    return std::any_of(std::cbegin(encryptionKeys), std::cend(encryptionKeys),
-                       [protocol] (const auto &protocolKeysMap) {
-                           return anyKeyHasProtocol(protocolKeysMap.value(UnknownProtocol), protocol);
-                       });
+    return std::any_of(std::cbegin(encryptionKeys), std::cend(encryptionKeys), [protocol](const auto &protocolKeysMap) {
+        return anyKeyHasProtocol(protocolKeysMap.value(UnknownProtocol), protocol);
+    });
 }
 
 auto keysForProtocol(const QMap<QString, QMap<Protocol, std::vector<Key>>> &encryptionKeys, Protocol protocol)
@@ -585,9 +596,9 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
     // check protocols needed for overrides
     const bool commonOverridesNeedOpenPGP = anyCommonOverrideHasKeyOfType(mEncKeys, OpenPGP);
     const bool commonOverridesNeedCMS = anyCommonOverrideHasKeyOfType(mEncKeys, CMS);
-    if ((mFormat == OpenPGP && commonOverridesNeedCMS)
-            || (mFormat == CMS && commonOverridesNeedOpenPGP)
-            || (!mAllowMixed && commonOverridesNeedOpenPGP && commonOverridesNeedCMS)) {
+    if ((mFormat == OpenPGP && commonOverridesNeedCMS) //
+        || (mFormat == CMS && commonOverridesNeedOpenPGP) //
+        || (!mAllowMixed && commonOverridesNeedOpenPGP && commonOverridesNeedCMS)) {
         // invalid protocol requirements -> clear intermediate result and abort resolution
         mEncKeys.clear();
         return {Error, {}, {}};
@@ -606,13 +617,14 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
         resolveSign(OpenPGP);
         resolveEnc(OpenPGP);
     }
-    const bool pgpOnly = (!mEncrypt || !hasUnresolvedRecipients(mEncKeys, OpenPGP)) && (!mSign || mSigKeys.contains(OpenPGP));
+    const bool pgpOnly = ((!mEncrypt || !hasUnresolvedRecipients(mEncKeys, OpenPGP)) //
+                          && (!mSign || !hasUnresolvedSender(mSigKeys, OpenPGP)));
 
     if (mFormat == OpenPGP) {
         return {
             SolutionFlags((pgpOnly ? AllResolved : SomeUnresolved) | OpenPGPOnly),
             {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
-            {}
+            {},
         };
     }
 
@@ -620,13 +632,14 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
         resolveSign(CMS);
         resolveEnc(CMS);
     }
-    const bool cmsOnly = (!mEncrypt || !hasUnresolvedRecipients(mEncKeys, CMS)) && (!mSign || mSigKeys.contains(CMS));
+    const bool cmsOnly = ((!mEncrypt || !hasUnresolvedRecipients(mEncKeys, CMS)) //
+                          && (!mSign || !hasUnresolvedSender(mSigKeys, CMS)));
 
     if (mFormat == CMS) {
         return {
             SolutionFlags((cmsOnly ? AllResolved : SomeUnresolved) | CMSOnly),
             {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
-            {}
+            {},
         };
     }
 
@@ -636,13 +649,13 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
             return {
                 SolutionFlags(AllResolved | CMSOnly),
                 {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
-                {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)}
+                {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
             };
         } else {
             return {
                 SolutionFlags(AllResolved | CMSOnly),
                 {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
-                {}
+                {},
             };
         }
     }
@@ -651,13 +664,13 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
             return {
                 SolutionFlags(AllResolved | OpenPGPOnly),
                 {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
-                {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)}
+                {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
             };
         } else {
             return {
                 SolutionFlags(AllResolved | OpenPGPOnly),
                 {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
-                {}
+                {},
             };
         }
     }
@@ -668,52 +681,57 @@ KeyResolverCore::Result KeyResolverCore::Private::resolve()
             return {
                 SolutionFlags(SomeUnresolved | CMSOnly),
                 {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
-                {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)}
+                {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
             };
         } else {
             return {
                 SolutionFlags(SomeUnresolved | OpenPGPOnly),
                 {OpenPGP, mSigKeys.value(OpenPGP), keysForProtocol(mEncKeys, OpenPGP)},
-                {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)}
+                {CMS, mSigKeys.value(CMS), keysForProtocol(mEncKeys, CMS)},
             };
         }
     }
 
     const auto bestEncryptionKeys = getBestEncryptionKeys(mEncKeys, mPreferredProtocol);
-    const bool allAddressesAreResolved = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys),
-                                                     [] (const auto &keys) { return !keys.empty(); });
-    if (allAddressesAreResolved) {
+    // we are in mixed mode, i.e. we need an OpenPGP signing key and an S/MIME signing key
+    const bool senderIsResolved = (!mSign || (!hasUnresolvedSender(mSigKeys, OpenPGP) && !hasUnresolvedSender(mSigKeys, CMS)));
+    const bool allRecipientsAreResolved = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys), [](const auto &keys) {
+        return !keys.empty();
+    });
+    if (senderIsResolved && allRecipientsAreResolved) {
         return {
             SolutionFlags(AllResolved | MixedProtocols),
             {UnknownProtocol, concatenate(mSigKeys.value(OpenPGP), mSigKeys.value(CMS)), bestEncryptionKeys},
-            {}
+            {},
         };
     }
 
-    const bool allKeysAreOpenPGP = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys),
-                                               [] (const auto &keys) { return allKeysHaveProtocol(keys, OpenPGP); });
+    const bool allKeysAreOpenPGP = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys), [](const auto &keys) {
+        return allKeysHaveProtocol(keys, OpenPGP);
+    });
     if (allKeysAreOpenPGP) {
         return {
             SolutionFlags(SomeUnresolved | OpenPGPOnly),
             {OpenPGP, mSigKeys.value(OpenPGP), bestEncryptionKeys},
-            {}
+            {},
         };
     }
 
-    const bool allKeysAreCMS = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys),
-                                           [] (const auto &keys) { return allKeysHaveProtocol(keys, CMS); });
+    const bool allKeysAreCMS = std::all_of(std::begin(bestEncryptionKeys), std::end(bestEncryptionKeys), [](const auto &keys) {
+        return allKeysHaveProtocol(keys, CMS);
+    });
     if (allKeysAreCMS) {
         return {
             SolutionFlags(SomeUnresolved | CMSOnly),
             {CMS, mSigKeys.value(CMS), bestEncryptionKeys},
-            {}
+            {},
         };
     }
 
     return {
         SolutionFlags(SomeUnresolved | MixedProtocols),
         {UnknownProtocol, concatenate(mSigKeys.value(OpenPGP), mSigKeys.value(CMS)), bestEncryptionKeys},
-        {}
+        {},
     };
 }
 
